package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.params.KeyParameter;
import cn.com.infosec.crypto.params.RC5Parameters;

public class RC532Engine implements BlockCipher {
   private static final int P32 = -1209970333;
   private static final int Q32 = -1640531527;
   private int _noRounds = 12;
   private int[] _S = null;
   private boolean forEncryption;

   private void setKey(byte[] param1) {
      // $FF: Couldn't be decompiled
   }

   private int encryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int in1 = this.bytesToWord(in, inOff) + this._S[0];
      inOff = this.bytesToWord(in, inOff + 4) + this._S[1];

      int var10000;
      for(int var5 = 1; var5 <= this._noRounds; in1 = var10000) {
         in1 = this.rotateLeft(inOff ^ (inOff = this.rotateLeft(in1 ^ inOff, inOff) + this._S[in1 = var5 * 2]), inOff) + this._S[in1 + 1];
         ++var5;
         var10000 = inOff;
         inOff = in1;
      }

      this.wordToBytes(in1, out, outOff);
      int var6 = outOff + 4;
      this.wordToBytes(inOff, out, var6);
      return 8;
   }

   private int decryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5 = this.bytesToWord(in, inOff);
      int in1 = this.bytesToWord(in, inOff + 4);

      int var10000;
      for(inOff = this._noRounds; inOff >= 1; in1 = var10000) {
         int var10001 = var5;
         var5 ^= this.rotateRight(in1 - this._S[(in1 = inOff * 2) + 1], var5);
         in1 = this.rotateRight(var10001 - this._S[in1], var5) ^ var5;
         --inOff;
         var10000 = var5;
         var5 = in1;
      }

      this.wordToBytes(var5 - this._S[0], out, outOff);
      this.wordToBytes(in1 - this._S[1], out, outOff + 4);
      return 8;
   }

   private int rotateLeft(int x, int y) {
      int var3;
      return x << (var3 = y & 31) | x >>> 32 - var3;
   }

   private int rotateRight(int x, int y) {
      int var3;
      return x >>> (var3 = y & 31) | x << 32 - var3;
   }

   private int bytesToWord(byte[] src, int srcOff) {
      return src[srcOff] & 255 | (src[srcOff + 1] & 255) << 8 | (src[srcOff + 2] & 255) << 16 | (src[srcOff + 3] & 255) << 24;
   }

   private void wordToBytes(int word, byte[] dst, int dstOff) {
      dst[dstOff] = (byte)word;
      int var4 = dstOff + 1;
      dst[var4] = (byte)(word >> 8);
      var4 = dstOff + 2;
      dst[var4] = (byte)(word >> 16);
      var4 = dstOff + 3;
      dst[var4] = (byte)(word >> 24);
   }

   public String getAlgorithmName() {
      return "RC5-32";
   }

   public int getBlockSize() {
      return 8;
   }

   public void init(boolean forEncryption, CipherParameters params) {
      if (params instanceof RC5Parameters) {
         RC5Parameters params1;
         this._noRounds = (params1 = (RC5Parameters)params).getRounds();
         this.setKey(params1.getKey());
      } else {
         if (!(params instanceof KeyParameter)) {
            throw new IllegalArgumentException("invalid parameter passed to RC532 init - " + params.getClass().getName());
         }

         this.setKey(((KeyParameter)params).getKey());
      }

      this.forEncryption = forEncryption;
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      return this.forEncryption ? this.encryptBlock(in, inOff, out, outOff) : this.decryptBlock(in, inOff, out, outOff);
   }

   public void reset() {
   }
}
